/*

     d888   8888888888     d88P 8888888888     d88P  .d8888b.  888888888
    d8888         d88P    d88P        d88P    d88P  d88P  Y88b 888
      888        d88P    d88P        d88P    d88P   888    888 888
      888       d88P    d88P        d88P    d88P    Y88b. d888 8888888b.
      888    88888888  d88P      88888888  d88P      "Y888P888      "Y88b
      888     d88P    d88P        d88P    d88P             888        888
      888    d88P    d88P        d88P    d88P       Y88b  d88P Y88b  d88P
    8888888 d88P    d88P        d88P    d88P         "Y8888P"   "Y8888P"

*/

/*** SIS_DEMO.C - Resolucao de sistemas nao lineares ***/

#include <stdio.h>
#include <math.h>

#define float long double

#include "sis.c"

/* ----------------------------------------------------------------------- */

/* Definicao do sistema  f(x, y, z) = 0
			 g(x, y, z) = 0
			 h(x, y, z) = 0   */

float f(float x, float y, float z) {

  return cos(x - 1)*exp(z - 3) - y + 1;
}

float g(float x, float y, float z) {

  return sin(x + 2*y + 3*z) - cos(z/3 - y/2)*sin(8*x + z + 3);
}

float h(float x, float y, float z) {

  return exp(cos(z*M_PI/2)) - z + x*y*exp((x + y)/3 - 1);
}

/* ----------------------------------------------------------------------- */

void main(void) {

  float x[3], x0[3];
  int n1, n2;

  x0[0] = 2.0; x0[1] = 2.0; x0[2] = 2.0;  /* "Chute" inicial da solucao */

  printf("\n\n================\n\nSistema:\n");
  printf("\n    {               (z - 3)");
  printf("\n    {   cos(x - 1) e      - y + 1 = 0");
  printf("\n    {");
  printf("\n    {    cos( z/2))                ((x + y)/3 - 1)");
  printf("\n    {   e                - z + x y e               = 0");
  printf("\n    {");
  printf("\n    {   sen(x + 2 y + 3 z) - cos(z/3 - y/2) sen(8 x + z + 3) = 0");
  printf("\n\nAproximacao inicial = (%Lf, %Lf, %Lf)", x0[0], x0[1], x0[2]);

  n1 = ResolveSistema(f, g, h, x0, 1E-6, 1000, x); /* Tenta  achar  uma */
						   /* solucao grosseira */
  if (n1 > 0) {
    IGUALAVETOR(x0, x);
    n2 = ResolveSistema(f, g, h, x0, 1E-12, 1000, x); /* Refinamento */
  }  						      /* da solucao  */

  if (n2 > 0) {
    printf("\n\nSolucao encontrada = (%15.10Lf, %15.10Lf, %15.10Lf)",
				  x[0], x[1], x[2]);
    printf("\n\nQuantidade de iteracoes utilizadas = %d", n1 + n2);
    printf("\n\nTeste = (%18.16Lf, %18.16Lf, %18.16Lf)\n\n",
	      f(x[0], x[1], x[2]), g(x[0], x[1], x[2]), h(x[0], x[1], x[2]));
  }
  else printf("\n\nSolucao nao pode ser encontrada com a aproximacao dada");
}

/* ----------------------------------------------------------------------- */

/*** FIM DE SIS_DEMO.C ***/
